@@ -13,7 +13,7 @@ use pyo3::{ffi, intern, AsPyPointer, PyTypeInfo};
 
 use crate::build_tools::safe_repr;
 use crate::errors::{ErrorType, InputValue, LocItem, ValError, ValLineError, ValResult};
-use crate::{PyMultiHostUrl, PyUrl};
+use crate::{ArgsKwargs, PyMultiHostUrl, PyUrl};
 
 use super::datetime::{
     bytes_as_date, bytes_as_datetime, bytes_as_time, bytes_as_timedelta, date_as_datetime, float_as_datetime,
@@ -40,7 +40,7 @@ macro_rules! extract_shared_iter {
     };
 }
 
-/// Extract dict keys, values and items into a `GenericCollection`, not available on PyPy
+/// Extract dict keys, values and items into a `GenericCollection`
 #[cfg(not(PyPy))]
 macro_rules! extract_dict_iter {
     ($obj:ident) => {
@@ -56,6 +56,21 @@ macro_rules! extract_dict_iter {
     };
 }
 
+#[cfg(PyPy)]
+macro_rules! extract_dict_iter {
+    ($obj:ident) => {
+        if is_dict_keys_type($obj) {
+            Some($obj.into())
+        } else if is_dict_values_type($obj) {
+            Some($obj.into())
+        } else if is_dict_items_type($obj) {
+            Some($obj.into())
+        } else {
+            None
+        }
+    };
+}
+
 impl<'a> Input<'a> for PyAny {
     fn get_type(&self) -> &'static InputType {
         &InputType::Python
@@ -106,6 +121,14 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
+    fn maybe_subclass_dict(&self, class: &PyType) -> PyResult<&Self> {
+        if matches!(self.is_instance(class), Ok(true)) {
+            self.getattr(intern!(self.py(), "__dict__"))
+        } else {
+            Ok(self)
+        }
+    }
+
     fn input_as_url(&self) -> Option<PyUrl> {
         self.extract::<PyUrl>().ok()
     }
@@ -120,8 +143,8 @@ impl<'a> Input<'a> for PyAny {
 
     fn validate_args(&'a self) -> ValResult<'a, GenericArguments<'a>> {
         if let Ok(dict) = self.downcast::<PyDict>() {
-            if let Some(args) = dict.get_item("__args__") {
-                if let Some(kwargs) = dict.get_item("__kwargs__") {
+            if let Some(args) = dict.get_item(intern!(self.py(), "__args__")) {
+                if let Some(kwargs) = dict.get_item(intern!(self.py(), "__kwargs__")) {
                     // we only try this logic if there are only these two items in the dict
                     if dict.len() == 2 {
                         let args = if let Ok(tuple) = args.downcast::<PyTuple>() {
@@ -129,7 +152,7 @@ impl<'a> Input<'a> for PyAny {
                         } else if args.is_none() {
                             Ok(None)
                         } else if let Ok(list) = args.downcast::<PyList>() {
-                            Ok(Some(PyTuple::new(self.py(), list.iter())))
+                            Ok(Some(list_as_tuple(list)))
                         } else {
                             Err(ValLineError::new_with_loc(
                                 ErrorType::PositionalArgumentsType,
@@ -165,13 +188,26 @@ impl<'a> Input<'a> for PyAny {
         } else if let Ok(tuple) = self.downcast::<PyTuple>() {
             Ok(PyArgs::new(Some(tuple), None).into())
         } else if let Ok(list) = self.downcast::<PyList>() {
-            let tuple = PyTuple::new(self.py(), list.iter());
+            let tuple = list_as_tuple(list);
             Ok(PyArgs::new(Some(tuple), None).into())
         } else {
             Err(ValError::new(ErrorType::ArgumentsType, self))
         }
     }
 
+    fn validate_dataclass_args(&'a self, dataclass_name: &str) -> ValResult<'a, GenericArguments<'a>> {
+        if let Ok(dict) = self.downcast::<PyDict>() {
+            Ok(PyArgs::new(None, Some(dict)).into())
+        } else if let Ok(args_kwargs) = self.extract::<ArgsKwargs>() {
+            let args = args_kwargs.args.into_ref(self.py());
+            let kwargs = args_kwargs.kwargs.map(|d| d.into_ref(self.py()));
+            Ok(PyArgs::new(Some(args), kwargs).into())
+        } else {
+            let dataclass_name = dataclass_name.to_string();
+            Err(ValError::new(ErrorType::DataclassType { dataclass_name }, self))
+        }
+    }
+
     fn parse_json(&'a self) -> ValResult<'a, JsonInput> {
         if let Ok(py_bytes) = self.downcast::<PyBytes>() {
             serde_json::from_slice(py_bytes.as_bytes()).map_err(|e| map_json_err(self, e))
@@ -373,7 +409,6 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    #[cfg(not(PyPy))]
     fn lax_list(&'a self, allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
         if let Ok(list) = self.downcast::<PyList>() {
             Ok(list.into())
@@ -390,21 +425,6 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    #[cfg(PyPy)]
-    fn lax_list(&'a self, allow_any_iter: bool) -> ValResult<GenericCollection<'a>> {
-        if let Ok(list) = self.downcast::<PyList>() {
-            Ok(list.into())
-        } else if let Ok(tuple) = self.downcast::<PyTuple>() {
-            Ok(tuple.into())
-        } else if allow_any_iter && self.iter().is_ok() {
-            Ok(self.into())
-        } else if let Some(collection) = extract_shared_iter!(PyList, self) {
-            Ok(collection)
-        } else {
-            Err(ValError::new(ErrorType::ListType, self))
-        }
-    }
-
     fn strict_tuple(&'a self) -> ValResult<GenericCollection<'a>> {
         if let Ok(tuple) = self.downcast::<PyTuple>() {
             Ok(tuple.into())
@@ -413,7 +433,6 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    #[cfg(not(PyPy))]
     fn lax_tuple(&'a self) -> ValResult<GenericCollection<'a>> {
         if let Ok(tuple) = self.downcast::<PyTuple>() {
             Ok(tuple.into())
@@ -428,19 +447,6 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    #[cfg(PyPy)]
-    fn lax_tuple(&'a self) -> ValResult<GenericCollection<'a>> {
-        if let Ok(tuple) = self.downcast::<PyTuple>() {
-            Ok(tuple.into())
-        } else if let Ok(list) = self.downcast::<PyList>() {
-            Ok(list.into())
-        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
-            Ok(collection)
-        } else {
-            Err(ValError::new(ErrorType::TupleType, self))
-        }
-    }
-
     fn strict_set(&'a self) -> ValResult<GenericCollection<'a>> {
         if let Ok(set) = self.downcast::<PySet>() {
             Ok(set.into())
@@ -449,7 +455,6 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    #[cfg(not(PyPy))]
     fn lax_set(&'a self) -> ValResult<GenericCollection<'a>> {
         if let Ok(set) = self.downcast::<PySet>() {
             Ok(set.into())
@@ -468,23 +473,6 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    #[cfg(PyPy)]
-    fn lax_set(&'a self) -> ValResult<GenericCollection<'a>> {
-        if let Ok(set) = self.downcast::<PySet>() {
-            Ok(set.into())
-        } else if let Ok(list) = self.downcast::<PyList>() {
-            Ok(list.into())
-        } else if let Ok(tuple) = self.downcast::<PyTuple>() {
-            Ok(tuple.into())
-        } else if let Ok(frozen_set) = self.downcast::<PyFrozenSet>() {
-            Ok(frozen_set.into())
-        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
-            Ok(collection)
-        } else {
-            Err(ValError::new(ErrorType::SetType, self))
-        }
-    }
-
     fn strict_frozenset(&'a self) -> ValResult<GenericCollection<'a>> {
         if let Ok(set) = self.downcast::<PyFrozenSet>() {
             Ok(set.into())
@@ -493,7 +481,6 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    #[cfg(not(PyPy))]
     fn lax_frozenset(&'a self) -> ValResult<GenericCollection<'a>> {
         if let Ok(frozen_set) = self.downcast::<PyFrozenSet>() {
             Ok(frozen_set.into())
@@ -512,23 +499,6 @@ impl<'a> Input<'a> for PyAny {
         }
     }
 
-    #[cfg(PyPy)]
-    fn lax_frozenset(&'a self) -> ValResult<GenericCollection<'a>> {
-        if let Ok(frozen_set) = self.downcast::<PyFrozenSet>() {
-            Ok(frozen_set.into())
-        } else if let Ok(set) = self.downcast::<PySet>() {
-            Ok(set.into())
-        } else if let Ok(list) = self.downcast::<PyList>() {
-            Ok(list.into())
-        } else if let Ok(tuple) = self.downcast::<PyTuple>() {
-            Ok(tuple.into())
-        } else if let Some(collection) = extract_shared_iter!(PyTuple, self) {
-            Ok(collection)
-        } else {
-            Err(ValError::new(ErrorType::FrozenSetType, self))
-        }
-    }
-
     fn validate_iter(&self) -> ValResult<GenericIterator> {
         if self.iter().is_ok() {
             Ok(self.into())
@@ -697,3 +667,66 @@ fn import_type(py: Python, module: &str, attr: &str) -> PyResult<Py<PyType>> {
 fn is_builtin_str(py_str: &PyString) -> bool {
     py_str.get_type().is(PyString::type_object(py_str.py()))
 }
+
+#[cfg(PyPy)]
+static DICT_KEYS_TYPE: GILOnceCell<Py<PyType>> = GILOnceCell::new();
+
+#[cfg(PyPy)]
+fn is_dict_keys_type(v: &PyAny) -> bool {
+    let py = v.py();
+    let keys_type = DICT_KEYS_TYPE
+        .get_or_init(py, || {
+            py.eval("type({}.keys())", None, None)
+                .unwrap()
+                .extract::<&PyType>()
+                .unwrap()
+                .into()
+        })
+        .as_ref(py);
+    v.is_instance(keys_type).unwrap_or(false)
+}
+
+#[cfg(PyPy)]
+static DICT_VALUES_TYPE: GILOnceCell<Py<PyType>> = GILOnceCell::new();
+
+#[cfg(PyPy)]
+fn is_dict_values_type(v: &PyAny) -> bool {
+    let py = v.py();
+    let values_type = DICT_VALUES_TYPE
+        .get_or_init(py, || {
+            py.eval("type({}.values())", None, None)
+                .unwrap()
+                .extract::<&PyType>()
+                .unwrap()
+                .into()
+        })
+        .as_ref(py);
+    v.is_instance(values_type).unwrap_or(false)
+}
+
+#[cfg(PyPy)]
+static DICT_ITEMS_TYPE: GILOnceCell<Py<PyType>> = GILOnceCell::new();
+
+#[cfg(PyPy)]
+fn is_dict_items_type(v: &PyAny) -> bool {
+    let py = v.py();
+    let items_type = DICT_ITEMS_TYPE
+        .get_or_init(py, || {
+            py.eval("type({}.items())", None, None)
+                .unwrap()
+                .extract::<&PyType>()
+                .unwrap()
+                .into()
+        })
+        .as_ref(py);
+    v.is_instance(items_type).unwrap_or(false)
+}
+
+pub fn list_as_tuple(list: &PyList) -> &PyTuple {
+    let py_tuple: Py<PyTuple> = unsafe {
+        let ptr = list.as_ptr();
+        let tuple_ptr = ffi::PyList_AsTuple(ptr);
+        Py::from_owned_ptr(list.py(), tuple_ptr)
+    };
+    py_tuple.into_ref(list.py())
+}
