@@ -2,7 +2,7 @@ use std::fmt;
 
 use pyo3::exceptions::{PyAttributeError, PyTypeError};
 use pyo3::prelude::*;
-use pyo3::types::{PyDict, PyList, PyString};
+use pyo3::types::{PyDict, PyList, PyMapping, PyString};
 
 use crate::build_tools::py_err;
 use crate::input::{JsonInput, JsonObject};
@@ -44,7 +44,7 @@ macro_rules! py_string {
 
 impl LookupKey {
     pub fn from_py(py: Python, value: &PyAny, alt_alias: Option<&str>) -> PyResult<Self> {
-        if let Ok(alias_py) = value.cast_as::<PyString>() {
+        if let Ok(alias_py) = value.downcast::<PyString>() {
             let alias: String = alias_py.extract()?;
             let alias_py: Py<PyString> = alias_py.into_py(py);
             match alt_alias {
@@ -57,12 +57,12 @@ impl LookupKey {
                 None => Ok(LookupKey::Simple(alias, alias_py)),
             }
         } else {
-            let list: &PyList = value.cast_as()?;
+            let list: &PyList = value.downcast()?;
             let first = match list.get_item(0) {
                 Ok(v) => v,
                 Err(_) => return py_err!("Lookup paths should have at least one element"),
             };
-            let mut locs: Vec<Path> = if first.cast_as::<PyString>().is_ok() {
+            let mut locs: Vec<Path> = if first.downcast::<PyString>().is_ok() {
                 // list of strings rather than list of lists
                 vec![Self::path_choice(list)?]
             } else {
@@ -95,7 +95,7 @@ impl LookupKey {
         }
     }
 
-    pub fn py_get_item<'data, 's>(&'s self, dict: &'data PyDict) -> PyResult<Option<(&'s str, &'data PyAny)>> {
+    pub fn py_get_dict_item<'data, 's>(&'s self, dict: &'data PyDict) -> PyResult<Option<(&'s str, &'data PyAny)>> {
         match self {
             LookupKey::Simple(key, py_key) => match dict.get_item(py_key) {
                 Some(value) => Ok(Some((key, value))),
@@ -124,7 +124,49 @@ impl LookupKey {
         }
     }
 
-    pub fn py_get_attr<'data, 's>(&'s self, obj: &'data PyAny) -> PyResult<Option<(&'s str, &'data PyAny)>> {
+    pub fn py_get_mapping_item<'data, 's>(
+        &'s self,
+        dict: &'data PyMapping,
+    ) -> PyResult<Option<(&'s str, &'data PyAny)>> {
+        match self {
+            LookupKey::Simple(key, py_key) => match dict.get_item(py_key) {
+                Ok(value) => Ok(Some((key, value))),
+                _ => Ok(None),
+            },
+            LookupKey::Choice(key1, key2, py_key1, py_key2) => match dict.get_item(py_key1) {
+                Ok(value) => Ok(Some((key1, value))),
+                _ => match dict.get_item(py_key2) {
+                    Ok(value) => Ok(Some((key2, value))),
+                    _ => Ok(None),
+                },
+            },
+            LookupKey::PathChoices(path_choices) => {
+                for path in path_choices {
+                    // iterate over the path and plug each value into the py_any from the last step, starting with dict
+                    // this could just be a loop but should be somewhat faster with a functional design
+                    if let Some(v) = path.iter().try_fold(dict as &PyAny, |d, loc| loc.py_get_item(d)) {
+                        // Successfully found an item, return it
+                        let key = path.first().unwrap().get_key();
+                        return Ok(Some((key, v)));
+                    }
+                }
+                // got to the end of path_choices, without a match, return None
+                Ok(None)
+            }
+        }
+    }
+
+    pub fn py_get_attr<'data, 's>(
+        &'s self,
+        obj: &'data PyAny,
+        kwargs: Option<&'data PyDict>,
+    ) -> PyResult<Option<(&'s str, &'data PyAny)>> {
+        if let Some(dict) = kwargs {
+            if let Ok(Some(item)) = self.py_get_dict_item(dict) {
+                return Ok(Some(item));
+            }
+        }
+
         match self {
             LookupKey::Simple(key, py_key) => match py_get_attrs(obj, py_key)? {
                 Some(value) => Ok(Some((key, value))),
@@ -236,7 +278,7 @@ fn path_to_string(path: &Path) -> String {
 
 impl PathItem {
     pub fn from_py(index: usize, obj: &PyAny) -> PyResult<Self> {
-        if let Ok(py_str_key) = obj.cast_as::<PyString>() {
+        if let Ok(py_str_key) = obj.downcast::<PyString>() {
             let str_key = py_str_key.to_str()?.to_string();
             Ok(Self::S(str_key, py_str_key.into()))
         } else {
@@ -251,7 +293,7 @@ impl PathItem {
 
     pub fn py_get_item<'a>(&self, py_any: &'a PyAny) -> Option<&'a PyAny> {
         // we definitely don't want to index strings, so explicitly omit this case
-        if py_any.cast_as::<PyString>().is_ok() {
+        if py_any.downcast::<PyString>().is_ok() {
             None
         } else {
             // otherwise, blindly try getitem on v since no better logic is realistic
@@ -270,7 +312,7 @@ impl PathItem {
         match self {
             Self::S(_, py_key) => {
                 // if obj is a dict, we want to use get_item, not getattr
-                if obj.cast_as::<PyDict>().is_ok() {
+                if obj.downcast::<PyDict>().is_ok() {
                     Ok(self.py_get_item(obj))
                 } else {
                     py_get_attrs(obj, py_key)
@@ -302,7 +344,7 @@ impl PathItem {
 
 /// wrapper around `getattr` that returns `Ok(None)` for attribute errors, but returns other errors
 /// We dont check `try_from_attributes` because that check was performed on the top level object before we got here
-fn py_get_attrs<'a, 'b>(obj: &'a PyAny, attr_name: &'b Py<PyString>) -> PyResult<Option<&'a PyAny>> {
+fn py_get_attrs<'a>(obj: &'a PyAny, attr_name: &Py<PyString>) -> PyResult<Option<&'a PyAny>> {
     match obj.getattr(attr_name.extract::<&PyString>(obj.py())?) {
         Ok(attr) => Ok(Some(attr)),
         Err(err) => {
