@@ -98,6 +98,8 @@ combined_serializer! {
         super::type_serializers::other::IsSubclassBuilder;
         super::type_serializers::other::CallableBuilder;
         super::type_serializers::definitions::DefinitionsBuilder;
+        super::type_serializers::dataclass::DataclassArgsBuilder;
+        super::type_serializers::dataclass::DataclassBuilder;
     }
     // `both` means the struct is added to both the `CombinedSerializer` enum and the match statement in
     // `find_serializer` so they can be used via a `type` str.
