@@ -1,8 +1,13 @@
 use idna::punycode::decode_to_string;
+use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
 use pyo3::types::PyDict;
 use url::Url;
 
+use crate::SchemaValidator;
+
+static SCHEMA_DEFINITION_URL: GILOnceCell<SchemaValidator> = GILOnceCell::new();
+
 #[pyclass(name = "Url", module = "pydantic_core._pydantic_core")]
 #[derive(Clone)]
 #[cfg_attr(debug_assertions, derive(Debug))]
@@ -20,8 +25,22 @@ impl PyUrl {
     }
 }
 
+fn build_schema_validator(py: Python, schema_type: &str) -> SchemaValidator {
+    let schema: &PyDict = PyDict::new(py);
+    schema.set_item("type", schema_type).unwrap();
+    SchemaValidator::py_new(py, schema, None).unwrap()
+}
+
 #[pymethods]
 impl PyUrl {
+    #[new]
+    pub fn py_new(py: Python, url: &PyAny) -> PyResult<Self> {
+        let schema_obj = SCHEMA_DEFINITION_URL
+            .get_or_init(py, || build_schema_validator(py, "url"))
+            .validate_python(py, url, None, None)?;
+        schema_obj.extract(py)
+    }
+
     #[getter]
     pub fn scheme(&self) -> &str {
         self.lib_url.scheme()
@@ -120,14 +139,24 @@ impl PyMultiHostUrl {
     }
 }
 
+static SCHEMA_DEFINITION_MULTI_HOST_URL: GILOnceCell<SchemaValidator> = GILOnceCell::new();
+
 #[pymethods]
 impl PyMultiHostUrl {
+    #[new]
+    pub fn py_new(py: Python, url: &PyAny) -> PyResult<Self> {
+        let schema_obj = SCHEMA_DEFINITION_MULTI_HOST_URL
+            .get_or_init(py, || build_schema_validator(py, "multi-host-url"))
+            .validate_python(py, url, None, None)?;
+        schema_obj.extract(py)
+    }
+
     #[getter]
     pub fn scheme(&self) -> &str {
         self.ref_url.scheme()
     }
 
-    pub fn hosts<'s, 'py>(&'s self, py: Python<'py>) -> PyResult<Vec<&'py PyDict>> {
+    pub fn hosts<'py>(&self, py: Python<'py>) -> PyResult<Vec<&'py PyDict>> {
         if let Some(extra_urls) = &self.extra_urls {
             let mut hosts = Vec::with_capacity(extra_urls.len() + 1);
             for url in extra_urls {
@@ -223,7 +252,7 @@ impl PyMultiHostUrl {
     }
 }
 
-fn host_to_dict<'a, 'b>(py: Python<'a>, lib_url: &'b Url) -> PyResult<&'a PyDict> {
+fn host_to_dict<'a>(py: Python<'a>, lib_url: &Url) -> PyResult<&'a PyDict> {
     let dict = PyDict::new(py);
     dict.set_item(
         "username",
