@@ -12,7 +12,14 @@ def test_chain():
     validator = SchemaValidator(
         {
             'type': 'chain',
-            'steps': [{'type': 'str'}, {'type': 'function', 'mode': 'plain', 'function': lambda v, info: Decimal(v)}],
+            'steps': [
+                {'type': 'str'},
+                {
+                    'type': 'function',
+                    'mode': 'plain',
+                    'function': {'type': 'general', 'function': lambda v, info: Decimal(v)},
+                },
+            ],
         }
     )
 
@@ -25,10 +32,26 @@ def test_chain_many():
         {
             'type': 'chain',
             'steps': [
-                {'type': 'function', 'mode': 'plain', 'function': lambda v, info: f'{v}-1'},
-                {'type': 'function', 'mode': 'plain', 'function': lambda v, info: f'{v}-2'},
-                {'type': 'function', 'mode': 'plain', 'function': lambda v, info: f'{v}-3'},
-                {'type': 'function', 'mode': 'plain', 'function': lambda v, info: f'{v}-4'},
+                {
+                    'type': 'function',
+                    'mode': 'plain',
+                    'function': {'type': 'general', 'function': lambda v, info: f'{v}-1'},
+                },
+                {
+                    'type': 'function',
+                    'mode': 'plain',
+                    'function': {'type': 'general', 'function': lambda v, info: f'{v}-2'},
+                },
+                {
+                    'type': 'function',
+                    'mode': 'plain',
+                    'function': {'type': 'general', 'function': lambda v, info: f'{v}-3'},
+                },
+                {
+                    'type': 'function',
+                    'mode': 'plain',
+                    'function': {'type': 'general', 'function': lambda v, info: f'{v}-4'},
+                },
             ],
         }
     )
@@ -64,7 +87,11 @@ def test_json(py_and_json: PyAndJson, input_value, expected):
             'type': 'chain',
             'steps': [
                 {'type': 'union', 'choices': [{'type': 'str'}, {'type': 'float'}]},
-                {'type': 'function', 'mode': 'plain', 'function': lambda v, info: Decimal(v)},
+                {
+                    'type': 'function',
+                    'mode': 'plain',
+                    'function': {'type': 'general', 'function': lambda v, info: Decimal(v)},
+                },
             ],
         }
     )
@@ -78,12 +105,24 @@ def test_flatten():
         {
             'type': 'chain',
             'steps': [
-                {'type': 'function', 'mode': 'plain', 'function': lambda v, info: f'{v}-1'},
+                {
+                    'type': 'function',
+                    'mode': 'plain',
+                    'function': {'type': 'general', 'function': lambda v, info: f'{v}-1'},
+                },
                 {
                     'type': 'chain',
                     'steps': [
-                        {'type': 'function', 'mode': 'plain', 'function': lambda v, info: f'{v}-2'},
-                        {'type': 'function', 'mode': 'plain', 'function': lambda v, info: f'{v}-3'},
+                        {
+                            'type': 'function',
+                            'mode': 'plain',
+                            'function': {'type': 'general', 'function': lambda v, info: f'{v}-2'},
+                        },
+                        {
+                            'type': 'function',
+                            'mode': 'plain',
+                            'function': {'type': 'general', 'function': lambda v, info: f'{v}-3'},
+                        },
                     ],
                 },
             ],
@@ -101,7 +140,16 @@ def test_chain_empty():
 
 def test_chain_one():
     validator = SchemaValidator(
-        {'type': 'chain', 'steps': [{'type': 'function', 'mode': 'plain', 'function': lambda v, info: f'{v}-1'}]}
+        {
+            'type': 'chain',
+            'steps': [
+                {
+                    'type': 'function',
+                    'mode': 'plain',
+                    'function': {'type': 'general', 'function': lambda v, info: f'{v}-1'},
+                }
+            ],
+        }
     )
     assert validator.validate_python('input') == 'input-1'
     assert validator.title == 'function-plain[<lambda>()]'
@@ -129,7 +177,7 @@ def test_ask():
                         'return_fields_set': True,
                         'fields': {'field_a': {'schema': {'type': 'str'}}},
                     },
-                    {'type': 'function', 'mode': 'plain', 'function': f},
+                    {'type': 'function', 'mode': 'plain', 'function': {'type': 'general', 'function': f}},
                 ],
             },
         }
