@@ -1,4 +1,3 @@
-import platform
 import re
 from collections import deque
 from typing import Any, Dict, Type
@@ -142,20 +141,8 @@ def test_tuple_var_len_kwargs(kwargs: Dict[str, Any], input_value, expected):
         ((1, 2, '3'), (1, 2, 3)),
         ([1, 2, '3'], (1, 2, 3)),
         (deque((1, 2, '3')), (1, 2, 3)),
-        pytest.param(
-            {1: 10, 2: 20, '3': '30'}.keys(),
-            (1, 2, 3),
-            marks=pytest.mark.skipif(
-                platform.python_implementation() == 'PyPy', reason='dict views not implemented in pyo3 for pypy'
-            ),
-        ),
-        pytest.param(
-            {1: 10, 2: 20, '3': '30'}.values(),
-            (10, 20, 30),
-            marks=pytest.mark.skipif(
-                platform.python_implementation() == 'PyPy', reason='dict views not implemented in pyo3 for pypy'
-            ),
-        ),
+        ({1: 10, 2: 20, '3': '30'}.keys(), (1, 2, 3)),
+        ({1: 10, 2: 20, '3': '30'}.values(), (10, 20, 30)),
         ({1: 10, 2: 20, '3': '30'}, Err('Input should be a valid tuple [type=tuple_type,')),
         ({1, 2, '3'}, Err('Input should be a valid tuple [type=tuple_type,')),
         (frozenset([1, 2, '3']), Err('Input should be a valid tuple [type=tuple_type,')),
@@ -467,7 +454,6 @@ def test_generator_error():
         v.validate_python(gen(True))
 
 
-@pytest.mark.skipif(platform.python_implementation() == 'PyPy', reason='dict views not implemented in pyo3 for pypy')
 @pytest.mark.parametrize(
     'input_value,items_schema,expected',
     [
